@@ -562,7 +562,10 @@ clients."
             (enabled? ,(enabled? service))
             (running ,(result->sexp (slot-ref service 'running)))
             (conflicts ,(map canonical-name (conflicts-with service)))
-            (last-respawns ,(slot-ref service 'last-respawns))))
+            (last-respawns ,(slot-ref service 'last-respawns))
+            ,@(if (slot-ref service 'one-shot?)
+                  '((one-shot? #t))
+                  '())))
 
 (define-method (result->sexp (service <service>))
   ;; Serialize SERVICE to an sexp.
@@ -709,27 +712,50 @@ results."
 set when starting a service."
   (environ))
 
-(define* (read-pid-file file #:key (max-delay 5))
+(define* (read-pid-file file #:key (max-delay 5)
+                        (validate-pid? #f))
   "Wait for MAX-DELAY seconds for FILE to show up, and read its content as a
 number.  Return #f if FILE was not created or does not contain a number;
-otherwise return the number that was read (a PID)."
+otherwise return the number that was read (a PID).
+
+When VALIDATE-PID? is true, succeed if and only if the number that was read is
+the PID of an existing process in the current PID namespace.  This test cannot
+be used if FILE might contain a PID from another PID namespace--i.e., the
+daemon writing FILE is running in a separate PID namespace."
   (define start (current-time))
+
   (let loop ()
+    (define (try-again)
+      (and (< (current-time) (+ start max-delay))
+           (begin
+             ;; FILE does not exist yet, so wait and try again.
+             ;; XXX: Ideally we would yield to the main event loop
+             ;; and/or use inotify.
+             (sleep 1)
+             (loop))))
+
     (catch 'system-error
       (lambda ()
-        (string->number
-         (string-trim-both
-          (call-with-input-file file get-string-all))))
+        (match (string->number
+                (string-trim-both
+                 (call-with-input-file file get-string-all)))
+          (#f
+           ;; If we didn't get an integer, it may be because the daemon didn't
+           ;; create FILE atomically and isn't done writing to it.  Try again.
+           (try-again))
+          ((? integer? pid)
+           ;; It's possible, though unlikely, that PID is not a valid PID, for
+           ;; instance because writes to FILE did not complete.  When
+           ;; VALIDATE-PID? is true, check that PID is valid in the current
+           ;; PID namespace.
+           (if (or (not validate-pid?)
+                   (catch-system-error (kill pid 0) #t))
+               pid
+               (try-again)))))
       (lambda args
         (let ((errno (system-error-errno args)))
           (if (= ENOENT errno)
-              (and (< (current-time) (+ start max-delay))
-                   (begin
-                     ;; FILE does not exist yet, so wait and try again.
-                     ;; XXX: Ideally we would yield to the main event loop
-                     ;; and/or use inotify.
-                     (sleep 1)
-                     (loop)))
+              (try-again)
               (apply throw args)))))))
 
 (define* (exec-command command
@@ -916,7 +942,8 @@ start."
                                         environment-variables)))
             (if pid-file
                 (match (read-pid-file pid-file
-                                      #:max-delay pid-file-timeout)
+                                      #:max-delay pid-file-timeout
+                                      #:validate-pid? #t)
                   (#f
                    (catch-system-error (kill pid SIGTERM))
                    #f)
