@@ -238,7 +238,9 @@ output port, and PROC's result is returned."
 
 (define* (display-version #:optional (program-name (program-name)))
   (local-output "~a (~a) ~a" program-name package-name Version)
-  (local-output (l10n "Copyright (C) 2018 the Shepherd authors
+  ;; TRANSLATORS: '~a' is a placeholder for the copyright year.
+  (local-output (l10n "Copyright (C) ~a the Shepherd authors") 2019)
+  (local-output (l10n "\
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.")))
@@ -252,12 +254,11 @@ There is NO WARRANTY, to the extent permitted by law.")))
     "Report the given error message to stderr in standard GNU error format."
     (syntax-case s ()
       ((_ (p message) args ...)
-       (string? (syntax->datum #'message))
+       (and (free-identifier=? #'p #'l10n)
+            (string? (syntax->datum #'message)))
 
-       (with-syntax ((message (string-append
-                               "~a: " (syntax->datum #'message) "~%")))
-         #'(format (current-error-port) message
-                   (program-name) args ...))))))
+       #'(format (current-error-port) "~a: ~a~%" (program-name)
+                 (format #f (l10n message) args ...))))))
 
 (define* (display-line message #:optional (port (current-output-port)))
   "Display MESSAGE followed by a newline to PORT."
